#include <stdio.h>
#include <stdlib.h>
#include <math.h> // NAN

typedef struct {
	int size; // numero di dati memorizzabili
	int n; // numero di dati memorizzati
	double* a;
} heap;

int heap_is_empty(heap* h)
{
	return h->n == 0;
}

int heap_is_full(heap* h)
{
	return h->n == h->size;
}

void heap_insert(heap *h, double val)
{
	if(heap_is_full(h)) {
		fprintf(stderr, "Memoria esaurita!\n");
		// si potrebbe usare realloc
		return;
	}
	int i = ++h->n; // prima cella disponibile
	// risalgo se possibile
	while(i>1 && val>h->a[i/2]) {
		h->a[i] = h->a[i/2];
		i /= 2;
	}
	h->a[i] = val;
}

double heap_del(heap *h)
{
	if(heap_is_empty(h)) return NAN;
	if(h->n == 1) {
		h->n  = 0;
		return h->a[1];
	}
	double val = h->a[h->n--]; //  il valore da spostare
	double tmp = h->a[1]; //  il valore estratto
	int i=1; // parto dalla radice
	while(2*i<h->n) {
		// vi sono entrambi i figli
		if(val>=h->a[i*2]) {
			if(val>=h->a[i*2+1]) {
				// ho trovato la posizione corretta
				h->a[i] = val;
				return tmp;
			}
			// scendo a destra
			h->a[i] = h->a[i*2+1];
			i = i*2 + 1;
		} else {
			if(h->a[i*2] < h->a[i*2+1]) {
				// scendo a destra
				h->a[i] = h->a[i*2+1];
				i = i*2 + 1;
			} else {
				// scendo a sinistra
				h->a[i] = h->a[i*2];
				i = i*2;
			}
		}
	}
	if(2*i==h->n) {
		// vi  solo il figlio sinistro
		if(val<h->a[i*2]) {
			// scendo a sinistra
			h->a[i] = h->a[i*2];
			i *= 2;
		}
	}
	// ho trovato la posizione corretta
	h->a[i] = val;
	return tmp;
}

/*
 * crea un heap vuoto che memorizza al pi size dati
 */
heap* make_heap(int size)
{
	heap *h = calloc(1, sizeof(heap));
	if(!h) return NULL;
	h->a = malloc(size * sizeof(*h->a));
	if(!h->a) return h;
	h->a--; // serve per poter accedere con indici 1:N invece che 0:N-1
	h->size = size;
	return h;
}

/*
le operazioni seguenti non sono tipiche di un heap tree, servono per
testare il programma
*/
void leggi(heap *h, FILE *f)
{
	double val;
	while(fscanf(f, "%lf", &val)==1) heap_insert(h, val);
}

// stampa il contenuto del vettore
void stampa(heap *h, FILE* f)
{
	int i;
	for(i=1; i<=h->n; i++) fprintf(f, "%f ", h->a[i]);
	fprintf(f, "\n");
}

void svuota(heap *h, FILE *f)
{
	while(!heap_is_empty(h)) fprintf(f, "%f ", heap_del(h));
	fprintf(f, "\n");
}

int main(int argc, char *argv[])
{
	heap* h = make_heap(argc>1 ? atoi(argv[1]) : 15); // memorizzo al massimo 15 dati
	leggi(h, stdin);
	stampa(h, stderr);
	svuota(h, stdout);
	return 0;
}
